﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Net.Sockets;
using System.Text.RegularExpressions;
using System.Threading;
using System.Windows.Forms;

namespace HuxBot
{
    class IRCBot
    {
        readonly string version = "HuxBot v1.27 build 53";
        Hashtable triggers;
        CultureInfo culture = CultureInfo.CurrentCulture;
        TcpClient IRCConnection;
        IRCConfig config;
        SendQueue sq;
        NetworkStream ns;
        StreamReader sr;
        StreamWriter sw;
        TimeCounter tc;
        Regex matcher;
        HuxUI uiForm;
        System.Timers.Timer rtimer;
        string pattern;
        char[] delimit = new char[] { ',', ' ' };
        bool saveall;
        public bool uiRunning;
        public bool uiRun;
        bool shouldRun;
        public IRCBot(IRCConfig config, bool save)
        {
            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);
            StartUI();
            this.tc = new TimeCounter();
            this.saveall = save;
            this.config = config;
            if (saveall)
                this.pattern = @"(https?|ftp)://[^ \n\t]+?";
            else
            {
                if (File.Exists(this.config.filterpath))
                {
                    string append = null;
                    this.pattern = @"https?://([a-zA-Z0-9/\.\-&:@%\?_]+?\.(png|jpe?g|gif)";
                    using (StreamReader hl = new StreamReader(this.config.filterpath))
                    {
                        string line;
                        while ((line = hl.ReadLine()) != null)
                        {
                            if (!line.StartsWith("&"))
                            {
                                line = Regex.Replace(line, @"\.", @"\.");
                                line = Regex.Replace(line, @"\?", @"\?");
                                this.pattern += "|" + line + @"[^ \n\t]+";
                            }
                            else
                            {
                                append += "|" + line.Substring(1);
                            }
                        }
                    }
                    this.pattern += ")";
                    if (append != null)
                        this.pattern = "(" + this.pattern + append + ")";
                }
                else
                {
                    Console.WriteLine("No custom filters found.");
                    this.pattern = @"https?://[a-zA-Z0-9/\.\-&:@%\?_]+?\.(png|jpe?g|gif)";
                }
            }
            this.matcher = new Regex(this.pattern);
            try
            {
                this.IRCConnection = new TcpClient(config.server, config.port);
            }
            catch
            {
                Console.WriteLine("Connection error!");
            }
            try
            {
                this.ns = IRCConnection.GetStream();
                this.sr = new StreamReader(ns);
                this.sw = new StreamWriter(ns);
                this.SendData("USER", "HuxBot +xB Befriending.without.mercy :HuxBot");
                this.SendData("NICK", config.nick);
            }
            catch
            {
                Console.WriteLine("Communication error!");
            }
            this.LoadTriggers(config.triggerpath);
            this.sq = new SendQueue(this);
            this.rtimer = new System.Timers.Timer(250000);
            this.rtimer.Elapsed += rtimer_Elapsed;
            this.IRCWork();
        }
        void rtimer_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
        {
            this.shouldRun = false;
            try
            {
                this.IRCConnection = new TcpClient(config.server, config.port);
            }
            catch
            {
                Console.WriteLine("Connection error!");
            }
            try
            {
                this.ns = IRCConnection.GetStream();
                this.sr = new StreamReader(ns);
                this.sw = new StreamWriter(ns);
                this.SendData("USER", config.nick + " +xB Befriending.without.mercy :HuxBot");
                this.SendData("NICK", config.nick);
            }
            catch
            {
                Console.WriteLine("Communication error!");
            }
            this.shouldRun = true;
            this.IRCWork();
        }
        private void SendData(string cmd, string param = null, string message = null)
        {
            if (uiRunning) uiForm.WriteTextToServer(cmd.ToUpper(), param, message);
            if (param != null)
                param = ' ' + param;
            if (message != null)
                param += " :" + message;
            this.sw.WriteLine(cmd.ToUpper() + param);
            this.sw.Flush();
            Console.WriteLine(cmd.ToUpper() + param);
        }
        public void Say(string target, string message)
        {
            this.SendData("PRIVMSG", target, message);
        }
        private void SendMaster(string text)
        {
            this.sq.Add(this.config.master, text);
        }
        private void WriteFile(string str, string sender)
        {
            foreach (Match found in this.matcher.Matches(str))
            {
                if ((this.config.ignorelist == null || !this.config.ignorelist.Contains(found.Value)) && !this.IsListed(found.Value))
                {
                    using (StreamWriter sw2 = new StreamWriter(this.config.filepath, true))
                        sw2.WriteLine(found.Value + ' ' + sender + ' ' + DateTime.Now.ToString("yyyy-MM-dd-HH:mm:ss"));
                    Console.WriteLine("\n========================================\n{0} has been saved\n========================================\n", found.Value);
                }
            }
        }
        private void LoadTriggers(string path)
        {
            this.triggers = new Hashtable();
            if (File.Exists(path))
            {
                using (StreamReader fs = new StreamReader(path))
                {
                    Console.WriteLine("Loading custom triggers...");
                    string line;
                    while ((line = fs.ReadLine()) != null)
                    {
                        try
                        {
                            string[] paramet = line.Split('|');
                            int Tim = 0;
                            if (paramet.Length > 2)
                            {
                                try
                                {
                                    Tim = Int32.Parse(paramet[2]);
                                }
                                catch (FormatException)
                                {
                                    Console.WriteLine("Incorrect cooldown");
                                    Tim = 0;
                                }
                            }
                            this.triggers.Add(paramet[0].ToLower(this.culture), new Trigger(this.tc, paramet[1], Tim));
                            Console.WriteLine("Trigger {0} loaded (cooldown {1} seconds)", paramet[0], Tim);
                        }
                        catch (IndexOutOfRangeException)
                        {
                            Console.WriteLine("Incorrect trigger. Skipping.");
                        }
                    }
                    Console.WriteLine("Custom triggers loaded.");
                }
            }
            else Console.WriteLine("No custom triggers loaded.");
        }
        private void IRCWork()
        {
            string[] ex;
            string data;
            shouldRun = true;
            string[] chans = this.config.chan.Split(this.delimit);
            while (shouldRun)
            {
                data = sr.ReadLine();
                Console.WriteLine(data);
                if (data.Split(' ')[0] == "ERROR")
                {
                    shouldRun = false;
                    continue;
                }
                IRCMessage msg = new IRCMessage(data);
                char[] charSeparator = new char[] { ' ' };
                if (uiRunning) uiForm.WriteTextFromServer(msg.formatted);
                if (msg.mode == "PING")
                {
                    this.SendData("PONG", msg.message);
                    this.rtimer = new System.Timers.Timer(250000);
                    this.rtimer.Elapsed += rtimer_Elapsed;
                    continue;
                }
                switch (msg.mode)
                {
                    case "PRIVMSG":
                        ex = msg.message.Split(charSeparator, 2);
                        string command = ex[0].ToLower(this.culture);
                        switch (ex.Length)
                        {
                            case 1:
                                if (msg.sender == this.config.master)
                                {
                                    switch (command)
                                    {
                                        case "!quit":
                                            this.SendData("QUIT", null, "Desu.");
                                            shouldRun = false;
                                            break;
                                        case "!q":
                                            this.SendData("QUIT", null, "Desu.");
                                            if (uiRunning) uiForm.Close();
                                            shouldRun = false;
                                            break;
                                        case "!sort":
                                            this.SortList();
                                            break;
                                        case "!html":
                                            this.GenerateHTMLlist();
                                            break;
                                        case "!triglist":
                                            this.SendMaster("Listing custom triggers:");
                                            foreach (DictionaryEntry entry in this.triggers)
                                            {
                                                this.SendMaster(" - " + entry.Key);
                                            }
                                            break;
                                        case "!reload":
                                            this.LoadTriggers(this.config.triggerpath);
                                            this.SendMaster("Reloading custom triggers.");
                                            break;
                                        case "!regex":
                                            this.SendMaster("Current regex pattern: " + this.pattern);
                                            break;
                                        case "!version":
                                            this.SendMaster(this.version);
                                            break;
                                        case "!loadui":
                                            if (!uiRunning) StartUI();
                                            break;
                                    }
                                }
                                if (triggers.ContainsKey(command))
                                {
                                    Console.WriteLine("Read trigger " + command);
                                    string ans = ((Trigger)(triggers[command])).Answer(msg.sender);
                                    if (ans != null)
                                        this.SendData("PRIVMSG", msg.target, ans);
                                }
                                else if (command == "!time")
                                {
                                    Console.WriteLine("Read trigger " + command);
                                    string ans = TimeTrigger.Answer();
                                    if (ans != null)
                                        this.SendData("PRIVMSG", msg.target, ans);
                                }
                                break;
                            case 2:
                                if (msg.sender == this.config.master)
                                {
                                    switch (command)
                                    {
                                        case "!part":
                                            SendData("PART", ":" + ex[1]);
                                            break;
                                        case "!p":
                                            SendData("PART", ":" + ex[1]);
                                            break;
                                        case "!join":
                                            SendData("JOIN", ex[1]);
                                            break;
                                        case "!j":
                                            SendData("JOIN", ex[1]);
                                            break;
                                        case "!say":
                                            string[] say = ex[1].Split(this.delimit, 2);
                                            SendData("PRIVMSG", say[0], say[1]);
                                            break;
                                        case "!me":
                                            string[] me = ex[1].Split(this.delimit, 2);
                                            SendData("PRIVMSG", me[0], (char)1 + "ACTION " + me[1] + (char)1);
                                            break;
                                        case "!s":
                                            string[] s = ex[1].Split(this.delimit, 2);
                                            SendData("PRIVMSG", s[0], s[1]);
                                            break;
                                        case "!quit":
                                            SendData("QUIT", null, ex[1]);
                                            shouldRun = false;
                                            break;
                                        case "!q":
                                            SendData("QUIT", null, ex[1]);
                                            shouldRun = false;
                                            break;
                                        case "!transfer":
                                            Console.WriteLine("Transferring ownership to " + ex[1]);
                                            this.SendMaster("Transferring ownership to " + ex[1]);
                                            this.config.master = ex[1];
                                            this.SendMaster("Ownership of " + this.config.name + " transferred.");
                                            break;
                                        case "!version":
                                            if (!ex[1].StartsWith("#"))
                                                this.SendData("PRIVMSG", ex[1], (char)1 + "VERSION" + (char)1);
                                            break;
                                        case "!html":
                                            this.GenerateHTMLlist(ex[1]);
                                            break;
                                    }
                                }
                                if (triggers.ContainsKey(command))
                                {
                                    Console.WriteLine("Read trigger " + command);
                                    string ans = ((Trigger)(triggers[command])).Stats(ex[1]);
                                    if (ans != null)
                                        this.SendData("PRIVMSG", msg.target, ans);
                                }
                                else if (command == "!time")
                                {
                                    Console.WriteLine("Read trigger " + command);
                                    string ans = TimeTrigger.Answer(ex[1]);
                                    if (ans != null)
                                        this.SendData("PRIVMSG", msg.target, ans);
                                }
                                break;
                        }
                        this.WriteFile(msg.message, msg.sender);
                        break;
                    case "376":
                        if (this.config.nspass != null && this.config.nspass != "")
                            this.SendData("NS", "IDENTIFY " + this.config.nspass);
                        this.SendData("JOIN", this.config.chan);
                        this.sq.Start();
                        break;
                    case "KICK":
                        if (msg.message.StartsWith(this.config.name))
                            this.SendData("JOIN", msg.target);
                        break;
                }
                if (msg.mode == "PRIVMSG")
                {
                    if (msg.sender != this.config.master && msg.sender != this.config.name && !(msg.target.StartsWith("#")))
                        this.sq.Add(this.config.master, (char)2 + "<" + msg.sender + "> " + (char)15 + msg.message);
                    else if (msg.sender == this.config.name)
                        this.sq.Add(this.config.master, msg.message);
                    if (msg.isCTCP)
                    {
                        string ctcp = msg.message.Substring(1).TrimEnd((char)1);
                        switch (ctcp.Split(' ')[0])
                        {
                            case "VERSION":
                                if (ctcp.Split(' ').Length == 1)
                                    this.SendData("NOTICE", msg.sender, (char)1 + "VERSION " + version + " [" + Environment.OSVersion + "] Current master: " + this.config.master + (char)1);
                                break;
                        }
                    }
                }
                if (msg.mode == "NOTICE")
                {
                    if (msg.sender != this.config.master && msg.sender != this.config.name && !(msg.target.StartsWith("#")))
                        this.sq.Add(this.config.master, (char)2 + "-" + msg.sender + "- " + (char)15 + msg.message);
                    else if (msg.sender == this.config.name)
                        this.sq.Add(this.config.master, msg.message);
                }
            }
        }
        private void StartUI()
        {
            uiForm = new HuxUI(this);
            Thread uit = new Thread(new UILoader(uiForm).Show);
            uit.Start();
            uiRunning = true;
            uiRun = true;
        }
        private void SortList()
        {
            List<string> entries = new List<string>();
            using (StreamReader sr2 = new StreamReader(this.config.filepath))
            {
                string readLine;
                while ((readLine = sr2.ReadLine()) != null)
                    entries.Add(readLine);
            }
            entries.Sort();
            FileStream fs = new FileStream(this.config.filepath.Substring(0, (this.config.filepath.Length - 4)) + "_sorted.txt", FileMode.Create, FileAccess.Write);
            using (StreamWriter sw2 = new StreamWriter(fs))
            {
                for (int i = 1; i < entries.Count; i++)
                    if (entries[i - 1].Split(' ')[0] != entries[i].Split(' ')[0])
                        sw2.WriteLine(entries[i].Split(' ')[0]);
            }
            this.SendMaster("Sorted list generated.");
        }
        private void GenerateHTMLlist(string type = "table")
        {
            List<string> entries = new List<string>();
            using (StreamReader sr2 = new StreamReader(this.config.filepath))
            {
                string readLine;
                while ((readLine = sr2.ReadLine()) != null)
                    if (!(entries.Contains(readLine)))
                        entries.Add(readLine);
            }
            entries.Sort();
            FileStream fs2 = new FileStream(this.config.filepath.Substring(0, (this.config.filepath.Length - 4)) + ".html", FileMode.Create, FileAccess.Write);
            using (StreamWriter sw2 = new StreamWriter(fs2))
            {
                switch (type.ToLower())
                {
                    case "table":
                        sw2.WriteLine(@"<table><tr><th>Date/time</th><th>URL</th><th>Posted by</th></tr>");
                        break;
                    case "list":
                        sw2.WriteLine(@"<ol>");
                        break;
                }
                for (int i = 1; i < entries.Count; i++)
                {
                    string[] line = entries[i].Split(' ');
                    if (line[0] != entries[i - 1].Split(' ')[0])
                    {
                        switch (type.ToLower())
                        {
                            case "table":
                                switch (line.Length)
                                {
                                    case 3:
                                        sw2.WriteLine("<tr><td>{0}</td><td><a href=\"{1}\" target=\"_blank\">{1}</a></td><td>{2}</td></tr>", line[2], line[0], line[1]);
                                        break;
                                    case 2:
                                        sw2.WriteLine("<tr><td></td><td><a href=\"{0}\" target=\"_blank\">{0}</a></td><td>{1}</td></tr>", line[0], line[1]);
                                        break;
                                    case 1:
                                        sw2.WriteLine("<tr><td></td><td><a href=\"{0}\" target=\"_blank\">{0}</td></tr>", line[0]);
                                        break;
                                }
                                break;
                            case "list":
                                switch (line.Length)
                                {
                                    case 3:
                                        sw2.WriteLine("<li><a href=\"{0}\" target=\"_blank\">{0}</a> from <b>{1}</b> on <b>{2}</b></li>", line[0], line[1], line[2]);
                                        break;
                                    case 2:
                                        sw2.WriteLine("<li><a href=\"{0}\" target=\"_blank\">{0}</a> from <b>{1}</b></li>", line[0], line[1]);
                                        break;
                                    case 1:
                                        sw2.WriteLine("<li><a href=\"{0}\" target=\"_blank\">{0}</a></li>", line[0]);
                                        break;
                                }
                                break;
                        }
                    }
                }
                switch (type.ToLower())
                {
                    case "table":
                        sw2.WriteLine(@"</table>");
                        break;
                    case "list":
                        sw2.WriteLine(@"</ol>");
                        break;
                }
            }
            this.SendMaster("HTML list generated.");
        }
        public void ParseFromUI(String destiny, String message)
        {
            char[] separator = { ' ' };
            String[] input = message.Split(' ');
            if (message.Length > 0 && input[0][0] == '/')
            {
                switch (input[0])
                {
                    case "/quit":
                        this.SendData("QUIT", null, "Desu.");
                        if (uiRunning)
                            uiForm.Close();
                        shouldRun = false;
                        break;
                    case "/q":
                        this.SendData("QUIT", null, "Desu.");
                        if (uiRunning)
                            uiForm.Close();
                        shouldRun = false;
                        break;
                    case "/sort":
                        this.SortList();
                        break;
                    case "/html":
                        this.GenerateHTMLlist();
                        break;
                    case "/triglist":
                        this.SendMaster("Listing custom triggers:");
                        foreach (DictionaryEntry entry in this.triggers)
                        {
                            this.SendMaster(" - " + entry.Key);
                        }
                        break;
                    case "/reload":
                        this.LoadTriggers(this.config.triggerpath);
                        this.SendMaster("Reloading custom triggers.");
                        break;
                    case "/regex":
                        this.SendMaster("Current regex pattern: " + this.pattern);
                        break;
                    case "/join":
                        for (int i = 1; i < input.Length; i++)
                        {
                            this.SendData("JOIN", input[i]);
                        }
                        break;
                    case "/j":
                        for (int i = 1; i < input.Length; i++)
                        {
                            this.SendData("JOIN", input[i]);
                        }
                        break;
                    case "/part":
                        if (input.Length == 2)
                            this.SendData("PART", input[1], null);
                        else if (input.Length > 2)
                            this.SendData("PART", input[1], message.Split(separator, 3)[2]);
                        break;
                    case "/p":
                        if (input.Length == 2)
                            this.SendData("PART", input[1], null);
                        else if (input.Length > 2)
                            this.SendData("PART", input[1], message.Split(separator, 3)[2]);
                        break;
                    case "/version":
                        if (input.Length == 2 && !input[1].StartsWith("#"))
                            this.SendData("PRIVMSG", input[1], (char)1 + "VERSION" + (char)1);
                        break;
                    default:
                        break;
                }
            }
            else if (destiny == null) return;
            else
            {
                this.SendData("PRIVMSG", destiny, message);
            }
        }
        private bool IsListed(string test)
        {
            try
            {
                using (StreamReader sr2 = new StreamReader(this.config.filepath))
                {
                    string line;
                    while ((line = sr2.ReadLine()) != null)
                        if (test == line.Split(' ')[0])
                            return true;
                    return false;
                }
            }
            catch
            {
                return false;
            }
        }
        public String GetNickname()
        {
            return config.nick;
        }
    }
}
